package com.example;

import bitronix.tm.resource.jdbc.PoolingDataSource;
import org.jbpm.runtime.manager.impl.DefaultRegisterableItemsFactory;
import org.jbpm.runtime.manager.impl.RuntimeEnvironmentBuilder;
import org.jbpm.services.task.audit.JPATaskLifeCycleEventListener;
import org.jbpm.services.task.identity.JBossUserGroupCallbackImpl;
import org.junit.AfterClass;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import org.kie.api.io.ResourceType;
import org.kie.api.runtime.manager.RuntimeEnvironment;
import org.kie.api.runtime.manager.RuntimeManager;
import org.kie.api.runtime.manager.RuntimeManagerFactory;
import org.kie.api.task.TaskLifeCycleEventListener;
import org.kie.internal.builder.KnowledgeBuilder;
import org.kie.internal.builder.KnowledgeBuilderFactory;
import org.kie.internal.io.ResourceFactory;
import org.kie.internal.task.api.UserGroupCallback;

import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import java.util.ArrayList;
import java.util.List;
import java.util.Properties;

public class MultipleRuntimeEngine {

    private static PoolingDataSource pds;
    private static EntityManagerFactory emf;

    @Test
    public void humanTaskTest() throws Exception {
        HumanTaskResolver htr1 = new HumanTaskResolver(1);
        HumanTaskResolver htr2 = new HumanTaskResolver(2);

        Thread t1 = new Thread(htr1, "first");
        Thread t2 = new Thread(htr2, "second");

        t1.start();
        t2.start();

        t1.join();
        t2.join();
    }

    class HumanTaskResolver implements Runnable {

        private final long pid;

        public HumanTaskResolver(long pid) {
            this.pid = pid;
        }

        @Override
        public void run() {
            RuntimeManager manager = getRuntimeManager("process/humantask/SimpleHumanTask.bpmn");
            manager.close();
        }

        private RuntimeManager getRuntimeManager(String process) {

            Properties properties = new Properties();
            properties.setProperty("some_group", "");

            UserGroupCallback userGroupCallback = new JBossUserGroupCallbackImpl(properties);
            // load up the knowledge base

            KnowledgeBuilder kbuilder = KnowledgeBuilderFactory.newKnowledgeBuilder();
            kbuilder.add(ResourceFactory.newClassPathResource(process), ResourceType.BPMN2);

            RuntimeEnvironment environment = RuntimeEnvironmentBuilder.getDefault()
                    .persistence(false)
                    .entityManagerFactory(emf)
                    .userGroupCallback(userGroupCallback)
                    .knowledgeBase(kbuilder.newKnowledgeBase())
                    .registerableItemsFactory(new DefaultRegisterableItemsFactory() {
                        @Override
                        public List<TaskLifeCycleEventListener> getTaskListeners() {
                            List<TaskLifeCycleEventListener> defaultListeners = new ArrayList<>();
                            defaultListeners.add(new JPATaskLifeCycleEventListener(false));
                            // add any custom listeners
                            defaultListeners.addAll(super.getTaskListeners());
                            // add listeners from deployment descriptor
                            defaultListeners.addAll(getTaskListenersFromDescriptor());

                            return defaultListeners;
                        }
                    })
                    .get();

            return RuntimeManagerFactory.Factory.get().newSingletonRuntimeManager(environment, "exec-" + pid);
        }
    }

    @BeforeClass
    public static void setup() {
        pds = new PoolingDataSource();
        pds.setUniqueName("jdbc/jbpm-ds");
        pds.setClassName("bitronix.tm.resource.jdbc.lrc.LrcXADataSource");
        pds.setMaxPoolSize(5);
        pds.setAllowLocalTransactions(true);
        pds.getDriverProperties().put("user", "jbpm6");
        pds.getDriverProperties().put("password", "jbpm6");
        pds.getDriverProperties().put("url", "jdbc:postgresql://localhost/jbpm6");
        pds.getDriverProperties().put("driverClassName", "org.postgresql.Driver");
        pds.init();

        Properties props = new Properties();
        props.setProperty("user", "jbpm6");
        props.setProperty("password", "jbpm6");
        props.setProperty("url", "jdbc:postgresql://localhost/jbpm6");
        props.setProperty("driverClassName", "org.postgresql.Driver");
        props.setProperty("hibernate.dialect", "org.hibernate.dialect.PostgreSQLDialect");
        emf = Persistence.createEntityManagerFactory("org.jbpm.persistence.jpa", props);
    }

    @AfterClass
    public static void tearDown() {
        pds.close();
        emf.close();
    }
}
